@@ -1,8 +1,8 @@
 CREATE TABLE IF NOT EXISTS radio_channels (
   source INTEGER NOT NULL DEFAULT 0,
-  name TEXT DEFAULT '',
-  url TEXT DEFAULT '',
-  thumbnail_url TEXT DEFAULT ''
+  name TEXT NOT NULL,
+  url TEXT NOT NULL,
+  thumbnail_url TEXT
 );
 
 UPDATE schema_version SET version=15;
