@@ -29,7 +29,7 @@
 #include <QVariant>
 #include <QString>
 
-#include "core/shared_ptr.h"
+#include "includes/shared_ptr.h"
 #include "core/filesystemmusicstorage.h"
 #include "core/iconloader.h"
 #include "core/musicstorage.h"
@@ -39,10 +39,11 @@
 #include "collectiondirectorymodel.h"
 
 using std::make_shared;
+using namespace Qt::Literals::StringLiterals;
 
 CollectionDirectoryModel::CollectionDirectoryModel(SharedPtr<CollectionBackend> backend, QObject *parent)
     : QStandardItemModel(parent),
-      dir_icon_(IconLoader::Load(QStringLiteral("document-open-folder"))),
+      dir_icon_(IconLoader::Load(u"document-open-folder"_s)),
       backend_(backend) {
 
   QObject::connect(&*backend_, &CollectionBackend::DirectoryAdded, this, &CollectionDirectoryModel::AddDirectory);
