@@ -31,7 +31,6 @@
 #include <QTextStream>
 #include <QFile>
 #include <QString>
-#include <QColor>
 #include <QPalette>
 #include <QEvent>
 
@@ -81,18 +80,11 @@ void StyleSheetLoader::UpdateStyleSheet(QWidget *widget, SharedPtr<StyleSheetDat
   QPalette p(widget->palette());
 
   {
-    QColor alt = p.color(QPalette::AlternateBase);
+    QColor color_altbase = p.color(QPalette::AlternateBase);
 #ifdef Q_OS_MACOS
-    if (alt.lightness() > 180) {
-      alt.setAlpha(130);
-    }
-    else {
-      alt.setAlpha(16);
-    }
-#else
-    alt.setAlpha(130);
+    color_altbase.setAlpha(color_altbase.lightness() > 180 ? 130 : 16);
 #endif
-    stylesheet.replace("%palette-alternate-base"_L1, QStringLiteral("rgba(%1,%2,%3,%4)").arg(alt.red()).arg(alt.green()).arg(alt.blue()).arg(alt.alpha()));
+    stylesheet.replace("%palette-alternate-base"_L1, QStringLiteral("rgba(%1,%2,%3,%4)").arg(color_altbase.red()).arg(color_altbase.green()).arg(color_altbase.blue()).arg(color_altbase.alpha()));
   }
 
   ReplaceColor(&stylesheet, u"Window"_s, p, QPalette::Window);
